// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use glib::{
    prelude::*,
    signal::{connect_raw, SignalHandlerId},
    translate::*,
};
use std::{boxed::Box as Box_, fmt, mem::transmute};

glib::wrapper! {
    #[doc(alias = "GtkStyleProvider")]
    pub struct StyleProvider(Interface<ffi::GtkStyleProvider>);

    match fn {
        type_ => || ffi::gtk_style_provider_get_type(),
    }
}

impl StyleProvider {
    pub const NONE: Option<&'static StyleProvider> = None;
}

pub trait StyleProviderExt: 'static {
    #[doc(alias = "gtk-private-changed")]
    fn connect_gtk_private_changed<F: Fn(&Self) + 'static>(&self, f: F) -> SignalHandlerId;
}

impl<O: IsA<StyleProvider>> StyleProviderExt for O {
    fn connect_gtk_private_changed<F: Fn(&Self) + 'static>(&self, f: F) -> SignalHandlerId {
        unsafe extern "C" fn gtk_private_changed_trampoline<
            P: IsA<StyleProvider>,
            F: Fn(&P) + 'static,
        >(
            this: *mut ffi::GtkStyleProvider,
            f: glib::ffi::gpointer,
        ) {
            let f: &F = &*(f as *const F);
            f(StyleProvider::from_glib_borrow(this).unsafe_cast_ref())
        }
        unsafe {
            let f: Box_<F> = Box_::new(f);
            connect_raw(
                self.as_ptr() as *mut _,
                b"gtk-private-changed\0".as_ptr() as *const _,
                Some(transmute::<_, unsafe extern "C" fn()>(
                    gtk_private_changed_trampoline::<Self, F> as *const (),
                )),
                Box_::into_raw(f),
            )
        }
    }
}

impl fmt::Display for StyleProvider {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        f.write_str("StyleProvider")
    }
}
