// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{Converter, FileInfo, ZlibCompressorFormat};
use glib::{
    prelude::*,
    signal::{connect_raw, SignalHandlerId},
    translate::*,
};
use std::{boxed::Box as Box_, fmt, mem::transmute};

glib::wrapper! {
    #[doc(alias = "GZlibDecompressor")]
    pub struct ZlibDecompressor(Object<ffi::GZlibDecompressor, ffi::GZlibDecompressorClass>) @implements Converter;

    match fn {
        type_ => || ffi::g_zlib_decompressor_get_type(),
    }
}

impl ZlibDecompressor {
    #[doc(alias = "g_zlib_decompressor_new")]
    pub fn new(format: ZlibCompressorFormat) -> ZlibDecompressor {
        unsafe { from_glib_full(ffi::g_zlib_decompressor_new(format.into_glib())) }
    }

    #[doc(alias = "g_zlib_decompressor_get_file_info")]
    #[doc(alias = "get_file_info")]
    pub fn file_info(&self) -> Option<FileInfo> {
        unsafe {
            from_glib_none(ffi::g_zlib_decompressor_get_file_info(
                self.to_glib_none().0,
            ))
        }
    }

    pub fn format(&self) -> ZlibCompressorFormat {
        glib::ObjectExt::property(self, "format")
    }

    #[doc(alias = "file-info")]
    pub fn connect_file_info_notify<F: Fn(&Self) + 'static>(&self, f: F) -> SignalHandlerId {
        unsafe extern "C" fn notify_file_info_trampoline<F: Fn(&ZlibDecompressor) + 'static>(
            this: *mut ffi::GZlibDecompressor,
            _param_spec: glib::ffi::gpointer,
            f: glib::ffi::gpointer,
        ) {
            let f: &F = &*(f as *const F);
            f(&from_glib_borrow(this))
        }
        unsafe {
            let f: Box_<F> = Box_::new(f);
            connect_raw(
                self.as_ptr() as *mut _,
                b"notify::file-info\0".as_ptr() as *const _,
                Some(transmute::<_, unsafe extern "C" fn()>(
                    notify_file_info_trampoline::<F> as *const (),
                )),
                Box_::into_raw(f),
            )
        }
    }
}

impl fmt::Display for ZlibDecompressor {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        f.write_str("ZlibDecompressor")
    }
}
