pub type DIFsrmClassificationEvents = *mut ::core::ffi::c_void;
pub type IFsrmAccessDeniedRemediationClient = *mut ::core::ffi::c_void;
pub type IFsrmAction = *mut ::core::ffi::c_void;
pub type IFsrmActionCommand = *mut ::core::ffi::c_void;
pub type IFsrmActionEmail = *mut ::core::ffi::c_void;
pub type IFsrmActionEmail2 = *mut ::core::ffi::c_void;
pub type IFsrmActionEventLog = *mut ::core::ffi::c_void;
pub type IFsrmActionReport = *mut ::core::ffi::c_void;
pub type IFsrmAutoApplyQuota = *mut ::core::ffi::c_void;
pub type IFsrmClassificationManager = *mut ::core::ffi::c_void;
pub type IFsrmClassificationManager2 = *mut ::core::ffi::c_void;
pub type IFsrmClassificationRule = *mut ::core::ffi::c_void;
pub type IFsrmClassifierModuleDefinition = *mut ::core::ffi::c_void;
pub type IFsrmClassifierModuleImplementation = *mut ::core::ffi::c_void;
pub type IFsrmCollection = *mut ::core::ffi::c_void;
pub type IFsrmCommittableCollection = *mut ::core::ffi::c_void;
pub type IFsrmDerivedObjectsResult = *mut ::core::ffi::c_void;
pub type IFsrmExportImport = *mut ::core::ffi::c_void;
pub type IFsrmFileCondition = *mut ::core::ffi::c_void;
pub type IFsrmFileConditionProperty = *mut ::core::ffi::c_void;
pub type IFsrmFileGroup = *mut ::core::ffi::c_void;
pub type IFsrmFileGroupImported = *mut ::core::ffi::c_void;
pub type IFsrmFileGroupManager = *mut ::core::ffi::c_void;
pub type IFsrmFileManagementJob = *mut ::core::ffi::c_void;
pub type IFsrmFileManagementJobManager = *mut ::core::ffi::c_void;
pub type IFsrmFileScreen = *mut ::core::ffi::c_void;
pub type IFsrmFileScreenBase = *mut ::core::ffi::c_void;
pub type IFsrmFileScreenException = *mut ::core::ffi::c_void;
pub type IFsrmFileScreenManager = *mut ::core::ffi::c_void;
pub type IFsrmFileScreenTemplate = *mut ::core::ffi::c_void;
pub type IFsrmFileScreenTemplateImported = *mut ::core::ffi::c_void;
pub type IFsrmFileScreenTemplateManager = *mut ::core::ffi::c_void;
pub type IFsrmMutableCollection = *mut ::core::ffi::c_void;
pub type IFsrmObject = *mut ::core::ffi::c_void;
pub type IFsrmPathMapper = *mut ::core::ffi::c_void;
pub type IFsrmPipelineModuleConnector = *mut ::core::ffi::c_void;
pub type IFsrmPipelineModuleDefinition = *mut ::core::ffi::c_void;
pub type IFsrmPipelineModuleImplementation = *mut ::core::ffi::c_void;
pub type IFsrmProperty = *mut ::core::ffi::c_void;
pub type IFsrmPropertyBag = *mut ::core::ffi::c_void;
pub type IFsrmPropertyBag2 = *mut ::core::ffi::c_void;
pub type IFsrmPropertyCondition = *mut ::core::ffi::c_void;
pub type IFsrmPropertyDefinition = *mut ::core::ffi::c_void;
pub type IFsrmPropertyDefinition2 = *mut ::core::ffi::c_void;
pub type IFsrmPropertyDefinitionValue = *mut ::core::ffi::c_void;
pub type IFsrmQuota = *mut ::core::ffi::c_void;
pub type IFsrmQuotaBase = *mut ::core::ffi::c_void;
pub type IFsrmQuotaManager = *mut ::core::ffi::c_void;
pub type IFsrmQuotaManagerEx = *mut ::core::ffi::c_void;
pub type IFsrmQuotaObject = *mut ::core::ffi::c_void;
pub type IFsrmQuotaTemplate = *mut ::core::ffi::c_void;
pub type IFsrmQuotaTemplateImported = *mut ::core::ffi::c_void;
pub type IFsrmQuotaTemplateManager = *mut ::core::ffi::c_void;
pub type IFsrmReport = *mut ::core::ffi::c_void;
pub type IFsrmReportJob = *mut ::core::ffi::c_void;
pub type IFsrmReportManager = *mut ::core::ffi::c_void;
pub type IFsrmReportScheduler = *mut ::core::ffi::c_void;
pub type IFsrmRule = *mut ::core::ffi::c_void;
pub type IFsrmSetting = *mut ::core::ffi::c_void;
pub type IFsrmStorageModuleDefinition = *mut ::core::ffi::c_void;
pub type IFsrmStorageModuleImplementation = *mut ::core::ffi::c_void;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdSyncTask: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x2ae64751_b728_4d6b_97a0_b2da2e7d2a3b);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_FEATURE_CLASSIFICATION: u32 = 83886080u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_FEATURE_FILESCREEN: u32 = 50331648u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_FEATURE_GENERAL: u32 = 16777216u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_FEATURE_MASK: u32 = 251658240u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_FEATURE_PIPELINE: u32 = 100663296u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_FEATURE_QUOTA: u32 = 33554432u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_FEATURE_REPORTS: u32 = 67108864u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_INTERFACE_A_MASK: u32 = 15728640u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_INTERFACE_B_MASK: u32 = 983040u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_INTERFACE_C_MASK: u32 = 61440u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_INTERFACE_D_MASK: u32 = 3840u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_IS_PROPERTY: u32 = 128u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_DISPID_METHOD_NUM_MASK: u32 = 127u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_ADR_MAX_EMAILS_SENT: ::windows_sys::core::HRESULT = -2147200130i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_ADR_NOT_DOMAIN_JOINED: ::windows_sys::core::HRESULT = -2147200110i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_ADR_PATH_IS_LOCAL: ::windows_sys::core::HRESULT = -2147200111i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_ADR_SRV_NOT_SUPPORTED: ::windows_sys::core::HRESULT = -2147200112i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_ALREADY_EXISTS: ::windows_sys::core::HRESULT = -2147200253i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_AUTO_QUOTA: ::windows_sys::core::HRESULT = 283419i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CACHE_INVALID: ::windows_sys::core::HRESULT = -2147200187i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CACHE_MODULE_ALREADY_EXISTS: ::windows_sys::core::HRESULT = -2147200186i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_AGGREGATE: ::windows_sys::core::HRESULT = -2147200201i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_ALLOW_REPARSE_POINT_TAG: ::windows_sys::core::HRESULT = -2147200170i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_CHANGE_PROPERTY_TYPE: ::windows_sys::core::HRESULT = -2147200197i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_CREATE_TEMP_COPY: ::windows_sys::core::HRESULT = -2147200132i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_DELETE_SYSTEM_PROPERTY: ::windows_sys::core::HRESULT = -2147200135i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_REMOVE_READONLY: ::windows_sys::core::HRESULT = -2147200109i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_RENAME_PROPERTY: ::windows_sys::core::HRESULT = -2147200198i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_STORE_PROPERTIES: ::windows_sys::core::HRESULT = -2147200171i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_USE_DELETED_PROPERTY: ::windows_sys::core::HRESULT = -2147200143i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CANNOT_USE_DEPRECATED_PROPERTY: ::windows_sys::core::HRESULT = -2147200145i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CLASSIFICATION_ALREADY_RUNNING: ::windows_sys::core::HRESULT = -2147200195i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CLASSIFICATION_CANCELED: ::windows_sys::core::HRESULT = -2147200141i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CLASSIFICATION_NOT_RUNNING: ::windows_sys::core::HRESULT = -2147200194i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CLASSIFICATION_PARTIAL_BATCH: ::windows_sys::core::HRESULT = -2147200136i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CLASSIFICATION_SCAN_FAIL: ::windows_sys::core::HRESULT = -2147200148i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CLASSIFICATION_TIMEOUT: ::windows_sys::core::HRESULT = -2147200137i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CLUSTER_NOT_RUNNING: ::windows_sys::core::HRESULT = -2147200210i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_CSC_PATH_NOT_SUPPORTED: ::windows_sys::core::HRESULT = -2147200106i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_DIFFERENT_CLUSTER_GROUP: ::windows_sys::core::HRESULT = -2147200207i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_DRIVER_NOT_READY: ::windows_sys::core::HRESULT = -2147200237i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_DUPLICATE_NAME: ::windows_sys::core::HRESULT = -2147200240i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_EMAIL_NOT_SENT: ::windows_sys::core::HRESULT = -2147200228i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_ENUM_PROPERTIES_FAILED: ::windows_sys::core::HRESULT = -2147200173i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_ERROR_NOT_ENABLED: ::windows_sys::core::HRESULT = -2147200133i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_EXPIRATION_PATH_NOT_WRITEABLE: ::windows_sys::core::HRESULT = -2147200105i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_EXPIRATION_PATH_TOO_LONG: ::windows_sys::core::HRESULT = -2147200104i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_EXPIRATION_VOLUME_NOT_NTFS: ::windows_sys::core::HRESULT = -2147200103i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FAIL_BATCH: ::windows_sys::core::HRESULT = -2147200247i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_ENCRYPTED: ::windows_sys::core::HRESULT = -2147200156i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_IN_USE: ::windows_sys::core::HRESULT = -2147200134i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_ACTION_GET_EXITCODE_FAILED: ::windows_sys::core::HRESULT = -2147200152i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_ACTION_TIMEOUT: ::windows_sys::core::HRESULT = -2147200153i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_EXPIRATION_DIR_IN_SCOPE: ::windows_sys::core::HRESULT = -2147200185i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_ALREADY_EXISTS: ::windows_sys::core::HRESULT = -2147200184i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_ALREADY_RUNNING: ::windows_sys::core::HRESULT = -2147200193i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_CUSTOM: ::windows_sys::core::HRESULT = -2147200191i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_DEPRECATED: ::windows_sys::core::HRESULT = -2147200102i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_EXPIRATION: ::windows_sys::core::HRESULT = -2147200192i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_INVALID_CONTINUOUS_CONFIG: ::windows_sys::core::HRESULT = -2147200108i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_MAX_FILE_CONDITIONS: ::windows_sys::core::HRESULT = -2147200146i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_NOTIFICATION: ::windows_sys::core::HRESULT = -2147200190i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_NOT_LEGACY_ACCESSIBLE: ::windows_sys::core::HRESULT = -2147200147i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_MANAGEMENT_JOB_RMS: ::windows_sys::core::HRESULT = -2147200120i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_OPEN_ERROR: ::windows_sys::core::HRESULT = -2147200189i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_FILE_SYSTEM_CORRUPT: ::windows_sys::core::HRESULT = -2147200225i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INCOMPATIBLE_FORMAT: ::windows_sys::core::HRESULT = -2147200157i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INPROC_MODULE_BLOCKED: ::windows_sys::core::HRESULT = -2147200174i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INSECURE_PATH: ::windows_sys::core::HRESULT = -2147200233i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INSUFFICIENT_DISK: ::windows_sys::core::HRESULT = -2147200236i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_AD_CLAIM: ::windows_sys::core::HRESULT = -2147200142i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_COMBINATION: ::windows_sys::core::HRESULT = -2147200241i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_DATASCREEN_DEFINITION: ::windows_sys::core::HRESULT = -2147200220i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_EMAIL_ADDRESS: ::windows_sys::core::HRESULT = -2147200226i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_FILEGROUP_DEFINITION: ::windows_sys::core::HRESULT = -2147200223i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_FILENAME: ::windows_sys::core::HRESULT = -2147200214i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_FOLDER_PROPERTY_STORE: ::windows_sys::core::HRESULT = -2147200140i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_IMPORT_VERSION: ::windows_sys::core::HRESULT = -2147200245i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_LIMIT: ::windows_sys::core::HRESULT = -2147200249i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_NAME: ::windows_sys::core::HRESULT = -2147200248i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_PATH: ::windows_sys::core::HRESULT = -2147200250i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_REPORT_DESC: ::windows_sys::core::HRESULT = -2147200215i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_REPORT_FORMAT: ::windows_sys::core::HRESULT = -2147200216i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_SCHEDULER_ARGUMENT: ::windows_sys::core::HRESULT = -2147200254i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_SMTP_SERVER: ::windows_sys::core::HRESULT = -2147200232i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_TEXT: ::windows_sys::core::HRESULT = -2147200246i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_INVALID_USER: ::windows_sys::core::HRESULT = -2147200251i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_LAST_ACCESS_UPDATE_DISABLED: ::windows_sys::core::HRESULT = -2147200176i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_LEGACY_SCHEDULE: ::windows_sys::core::HRESULT = -2147200107i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_LOADING_DISABLED_MODULE: ::windows_sys::core::HRESULT = -2147200202i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_LONG_CMDLINE: ::windows_sys::core::HRESULT = -2147200224i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_MAX_PROPERTY_DEFINITIONS: ::windows_sys::core::HRESULT = -2147200196i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_MESSAGE_LIMIT_EXCEEDED: ::windows_sys::core::HRESULT = -2147200200i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_MODULE_INITIALIZATION: ::windows_sys::core::HRESULT = -2147200150i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_MODULE_INVALID_PARAM: ::windows_sys::core::HRESULT = -2147200151i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_MODULE_SESSION_INITIALIZATION: ::windows_sys::core::HRESULT = -2147200149i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_MODULE_TIMEOUT: ::windows_sys::core::HRESULT = -2147200101i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_NOT_CLUSTER_VOLUME: ::windows_sys::core::HRESULT = -2147200208i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_NOT_FOUND: ::windows_sys::core::HRESULT = -2147200255i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_NOT_SUPPORTED: ::windows_sys::core::HRESULT = -2147200239i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_NO_EMAIL_ADDRESS: ::windows_sys::core::HRESULT = -2147200131i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_NO_PROPERTY_VALUE: ::windows_sys::core::HRESULT = -2147200175i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_OBJECT_IN_USE: ::windows_sys::core::HRESULT = -2147200199i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_OUT_OF_RANGE: ::windows_sys::core::HRESULT = -2147200243i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PARTIAL_CLASSIFICATION_PROPERTY_NOT_FOUND: ::windows_sys::core::HRESULT = -2147200169i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PATH_NOT_FOUND: ::windows_sys::core::HRESULT = -2147200252i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PATH_NOT_IN_NAMESPACE: ::windows_sys::core::HRESULT = -2147200129i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PERSIST_PROPERTIES_FAILED: ::windows_sys::core::HRESULT = -2147200155i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PERSIST_PROPERTIES_FAILED_ENCRYPTED: ::windows_sys::core::HRESULT = -2147200166i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PROPERTY_DELETED: ::windows_sys::core::HRESULT = -2147200183i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PROPERTY_MUST_APPLY_TO_FILES: ::windows_sys::core::HRESULT = -2147200138i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PROPERTY_MUST_APPLY_TO_FOLDERS: ::windows_sys::core::HRESULT = -2147200124i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PROPERTY_MUST_BE_GLOBAL: ::windows_sys::core::HRESULT = -2147200122i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_PROPERTY_MUST_BE_SECURE: ::windows_sys::core::HRESULT = -2147200123i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_REBUILDING_FODLER_TYPE_INDEX: ::windows_sys::core::HRESULT = -2147200139i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_REPORT_GENERATION_ERR: ::windows_sys::core::HRESULT = -2147200204i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_REPORT_JOB_ALREADY_RUNNING: ::windows_sys::core::HRESULT = -2147200205i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_REPORT_TASK_TRIGGER: ::windows_sys::core::HRESULT = -2147200203i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_REPORT_TYPE_ALREADY_EXISTS: ::windows_sys::core::HRESULT = -2147200206i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_REQD_PARAM_MISSING: ::windows_sys::core::HRESULT = -2147200242i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_RMS_NO_PROTECTORS_INSTALLED: ::windows_sys::core::HRESULT = -2147200126i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_RMS_NO_PROTECTOR_INSTALLED_FOR_FILE: ::windows_sys::core::HRESULT = -2147200125i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_RMS_TEMPLATE_NOT_FOUND: ::windows_sys::core::HRESULT = -2147200128i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_SECURE_PROPERTIES_NOT_SUPPORTED: ::windows_sys::core::HRESULT = -2147200127i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_SET_PROPERTY_FAILED: ::windows_sys::core::HRESULT = -2147200172i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_SHADOW_COPY: ::windows_sys::core::HRESULT = -2147200212i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_STORE_NOT_INSTALLED: ::windows_sys::core::HRESULT = -2147200209i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_SYNC_TASK_HAD_ERRORS: ::windows_sys::core::HRESULT = -2147200119i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_SYNC_TASK_TIMEOUT: ::windows_sys::core::HRESULT = -2147200144i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_TEXTREADER_FILENAME_TOO_LONG: ::windows_sys::core::HRESULT = -2147200158i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_TEXTREADER_IFILTER_CLSID_MALFORMED: ::windows_sys::core::HRESULT = -2147200160i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_TEXTREADER_IFILTER_NOT_FOUND: ::windows_sys::core::HRESULT = -2147200167i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_TEXTREADER_NOT_INITIALIZED: ::windows_sys::core::HRESULT = -2147200168i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_TEXTREADER_STREAM_ERROR: ::windows_sys::core::HRESULT = -2147200159i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_UNEXPECTED: ::windows_sys::core::HRESULT = -2147200234i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_UNSECURE_LINK_TO_HOSTED_MODULE: ::windows_sys::core::HRESULT = -2147200188i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_VOLUME_OFFLINE: ::windows_sys::core::HRESULT = -2147200154i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_VOLUME_UNSUPPORTED: ::windows_sys::core::HRESULT = -2147200235i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_WMI_FAILURE: ::windows_sys::core::HRESULT = -2147200121i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_E_XML_CORRUPTED: ::windows_sys::core::HRESULT = -2147200211i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_S_CLASSIFICATION_SCAN_FAILURES: ::windows_sys::core::HRESULT = 283398i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_S_PARTIAL_BATCH: ::windows_sys::core::HRESULT = 283396i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FSRM_S_PARTIAL_CLASSIFICATION: ::windows_sys::core::HRESULT = 283397i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmAccessDeniedRemediationClient: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x100b4fc8_74c1_470f_b1b7_dd7b6bae79bd);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmClassificationManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xb15c0e47_c391_45b9_95c8_eb596c853f3a);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmDaysNotSpecified: i32 = -1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmExportImport: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x1482dc37_fae9_4787_9025_8ce4e024ab56);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileGroupManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x8f1363f6_656f_4496_9226_13aecbd7718f);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementJobManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xeb18f9b2_4c3a_4321_b203_205120cff614);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileScreenManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x95941183_db53_4c5f_b37b_7d0921cf9dc7);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileScreenTemplateManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x243111df_e474_46aa_a054_eaa33edc292a);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmMaxExcludeFolders: u32 = 32u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmMaxNumberPropertyDefinitions: u32 = 100u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmMaxNumberThresholds: u32 = 16u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmMaxThresholdValue: u32 = 250u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmMinQuotaLimit: u32 = 1024u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmMinThresholdValue: u32 = 1u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPathMapper: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xf3be42bd_8ac2_409e_bbd8_faf9b6b41feb);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPipelineModuleConnector: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xc7643375_1eb5_44de_a062_623547d933bc);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmQuotaManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x90dcab7f_347c_4bfc_b543_540326305fbe);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmQuotaTemplateManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x97d3d443_251c_4337_81e7_b32e8f4ee65e);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportManager: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0x0058ef37_aa66_4c48_bd5b_2fce432ab0c8);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportScheduler: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xea25f1b8_1b8d_4290_8ee8_e17c12c2fe20);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmSetting: ::windows_sys::core::GUID = ::windows_sys::core::GUID::from_u128(0xf556d708_6d4d_4594_9c61_7dbb0dae2a46);
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const MessageSizeLimit: u32 = 4096u32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type AdrClientDisplayFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientDisplayFlags_AllowEmailRequests: AdrClientDisplayFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientDisplayFlags_ShowDeviceTroubleshooting: AdrClientDisplayFlags = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type AdrClientErrorType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientErrorType_Unknown: AdrClientErrorType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientErrorType_AccessDenied: AdrClientErrorType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientErrorType_FileNotFound: AdrClientErrorType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type AdrClientFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientFlags_None: AdrClientFlags = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientFlags_FailForLocalPaths: AdrClientFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientFlags_FailIfNotSupportedByServer: AdrClientFlags = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrClientFlags_FailIfNotDomainJoined: AdrClientFlags = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type AdrEmailFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrEmailFlags_PutDataOwnerOnToLine: AdrEmailFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrEmailFlags_PutAdminOnToLine: AdrEmailFlags = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrEmailFlags_IncludeDeviceClaims: AdrEmailFlags = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrEmailFlags_IncludeUserInfo: AdrEmailFlags = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const AdrEmailFlags_GenerateEventLog: AdrEmailFlags = 16i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmAccountType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmAccountType_Unknown: FsrmAccountType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmAccountType_NetworkService: FsrmAccountType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmAccountType_LocalService: FsrmAccountType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmAccountType_LocalSystem: FsrmAccountType = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmAccountType_InProc: FsrmAccountType = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmAccountType_External: FsrmAccountType = 5i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmAccountType_Automatic: FsrmAccountType = 500i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmActionType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmActionType_Unknown: FsrmActionType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmActionType_EventLog: FsrmActionType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmActionType_Email: FsrmActionType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmActionType_Command: FsrmActionType = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmActionType_Report: FsrmActionType = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmClassificationLoggingFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmClassificationLoggingFlags_None: FsrmClassificationLoggingFlags = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmClassificationLoggingFlags_ClassificationsInLogFile: FsrmClassificationLoggingFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmClassificationLoggingFlags_ErrorsInLogFile: FsrmClassificationLoggingFlags = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmClassificationLoggingFlags_ClassificationsInSystemLog: FsrmClassificationLoggingFlags = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmClassificationLoggingFlags_ErrorsInSystemLog: FsrmClassificationLoggingFlags = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmCollectionState = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmCollectionState_Fetching: FsrmCollectionState = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmCollectionState_Committing: FsrmCollectionState = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmCollectionState_Complete: FsrmCollectionState = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmCollectionState_Cancelled: FsrmCollectionState = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmCommitOptions = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmCommitOptions_None: FsrmCommitOptions = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmCommitOptions_Asynchronous: FsrmCommitOptions = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmEnumOptions = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEnumOptions_None: FsrmEnumOptions = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEnumOptions_Asynchronous: FsrmEnumOptions = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEnumOptions_CheckRecycleBin: FsrmEnumOptions = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEnumOptions_IncludeClusterNodes: FsrmEnumOptions = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEnumOptions_IncludeDeprecatedObjects: FsrmEnumOptions = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmEventType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEventType_Unknown: FsrmEventType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEventType_Information: FsrmEventType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEventType_Warning: FsrmEventType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmEventType_Error: FsrmEventType = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmExecutionOption = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmExecutionOption_Unknown: FsrmExecutionOption = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmExecutionOption_EvaluateUnset: FsrmExecutionOption = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmExecutionOption_ReEvaluate_ConsiderExistingValue: FsrmExecutionOption = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmExecutionOption_ReEvaluate_IgnoreExistingValue: FsrmExecutionOption = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmFileConditionType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileConditionType_Unknown: FsrmFileConditionType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileConditionType_Property: FsrmFileConditionType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmFileManagementLoggingFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementLoggingFlags_None: FsrmFileManagementLoggingFlags = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementLoggingFlags_Error: FsrmFileManagementLoggingFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementLoggingFlags_Information: FsrmFileManagementLoggingFlags = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementLoggingFlags_Audit: FsrmFileManagementLoggingFlags = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmFileManagementType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementType_Unknown: FsrmFileManagementType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementType_Expiration: FsrmFileManagementType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementType_Custom: FsrmFileManagementType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileManagementType_Rms: FsrmFileManagementType = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmFileScreenFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileScreenFlags_Enforce: FsrmFileScreenFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmFileStreamingInterfaceType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileStreamingInterfaceType_Unknown: FsrmFileStreamingInterfaceType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileStreamingInterfaceType_ILockBytes: FsrmFileStreamingInterfaceType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileStreamingInterfaceType_IStream: FsrmFileStreamingInterfaceType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmFileStreamingMode = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileStreamingMode_Unknown: FsrmFileStreamingMode = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileStreamingMode_Read: FsrmFileStreamingMode = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileStreamingMode_Write: FsrmFileStreamingMode = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmFileSystemPropertyId = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileSystemPropertyId_Undefined: FsrmFileSystemPropertyId = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileSystemPropertyId_FileName: FsrmFileSystemPropertyId = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileSystemPropertyId_DateCreated: FsrmFileSystemPropertyId = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileSystemPropertyId_DateLastAccessed: FsrmFileSystemPropertyId = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileSystemPropertyId_DateLastModified: FsrmFileSystemPropertyId = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmFileSystemPropertyId_DateNow: FsrmFileSystemPropertyId = 5i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmGetFilePropertyOptions = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmGetFilePropertyOptions_None: FsrmGetFilePropertyOptions = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmGetFilePropertyOptions_NoRuleEvaluation: FsrmGetFilePropertyOptions = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmGetFilePropertyOptions_Persistent: FsrmGetFilePropertyOptions = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmGetFilePropertyOptions_FailOnPersistErrors: FsrmGetFilePropertyOptions = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmGetFilePropertyOptions_SkipOrphaned: FsrmGetFilePropertyOptions = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPipelineModuleType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPipelineModuleType_Unknown: FsrmPipelineModuleType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPipelineModuleType_Storage: FsrmPipelineModuleType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPipelineModuleType_Classifier: FsrmPipelineModuleType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPropertyBagField = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyBagField_AccessVolume: FsrmPropertyBagField = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyBagField_VolumeGuidName: FsrmPropertyBagField = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPropertyBagFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyBagFlags_UpdatedByClassifier: FsrmPropertyBagFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyBagFlags_FailedLoadingProperties: FsrmPropertyBagFlags = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyBagFlags_FailedSavingProperties: FsrmPropertyBagFlags = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyBagFlags_FailedClassifyingProperties: FsrmPropertyBagFlags = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPropertyConditionType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_Unknown: FsrmPropertyConditionType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_Equal: FsrmPropertyConditionType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_NotEqual: FsrmPropertyConditionType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_GreaterThan: FsrmPropertyConditionType = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_LessThan: FsrmPropertyConditionType = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_Contain: FsrmPropertyConditionType = 5i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_Exist: FsrmPropertyConditionType = 6i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_NotExist: FsrmPropertyConditionType = 7i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_StartWith: FsrmPropertyConditionType = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_EndWith: FsrmPropertyConditionType = 9i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_ContainedIn: FsrmPropertyConditionType = 10i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_PrefixOf: FsrmPropertyConditionType = 11i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_SuffixOf: FsrmPropertyConditionType = 12i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyConditionType_MatchesPattern: FsrmPropertyConditionType = 13i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPropertyDefinitionAppliesTo = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionAppliesTo_Files: FsrmPropertyDefinitionAppliesTo = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionAppliesTo_Folders: FsrmPropertyDefinitionAppliesTo = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPropertyDefinitionFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionFlags_Global: FsrmPropertyDefinitionFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionFlags_Deprecated: FsrmPropertyDefinitionFlags = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionFlags_Secure: FsrmPropertyDefinitionFlags = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPropertyDefinitionType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_Unknown: FsrmPropertyDefinitionType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_OrderedList: FsrmPropertyDefinitionType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_MultiChoiceList: FsrmPropertyDefinitionType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_SingleChoiceList: FsrmPropertyDefinitionType = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_String: FsrmPropertyDefinitionType = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_MultiString: FsrmPropertyDefinitionType = 5i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_Int: FsrmPropertyDefinitionType = 6i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_Bool: FsrmPropertyDefinitionType = 7i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyDefinitionType_Date: FsrmPropertyDefinitionType = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPropertyFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_None: FsrmPropertyFlags = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_Orphaned: FsrmPropertyFlags = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_RetrievedFromCache: FsrmPropertyFlags = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_RetrievedFromStorage: FsrmPropertyFlags = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_SetByClassifier: FsrmPropertyFlags = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_Deleted: FsrmPropertyFlags = 16i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_Reclassified: FsrmPropertyFlags = 32i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_AggregationFailed: FsrmPropertyFlags = 64i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_Existing: FsrmPropertyFlags = 128i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_FailedLoadingProperties: FsrmPropertyFlags = 256i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_FailedClassifyingProperties: FsrmPropertyFlags = 512i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_FailedSavingProperties: FsrmPropertyFlags = 1024i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_Secure: FsrmPropertyFlags = 2048i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_PolicyDerived: FsrmPropertyFlags = 4096i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_Inherited: FsrmPropertyFlags = 8192i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_Manual: FsrmPropertyFlags = 16384i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_ExplicitValueDeleted: FsrmPropertyFlags = 32768i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_PropertyDeletedFromClear: FsrmPropertyFlags = 65536i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_PropertySourceMask: FsrmPropertyFlags = 14i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyFlags_PersistentMask: FsrmPropertyFlags = 20480i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmPropertyValueType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyValueType_Undefined: FsrmPropertyValueType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyValueType_Literal: FsrmPropertyValueType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmPropertyValueType_DateOffset: FsrmPropertyValueType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmQuotaFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmQuotaFlags_Enforce: FsrmQuotaFlags = 256i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmQuotaFlags_Disable: FsrmQuotaFlags = 512i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmQuotaFlags_StatusIncomplete: FsrmQuotaFlags = 65536i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmQuotaFlags_StatusRebuilding: FsrmQuotaFlags = 131072i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmReportFilter = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFilter_MinSize: FsrmReportFilter = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFilter_MinAgeDays: FsrmReportFilter = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFilter_MaxAgeDays: FsrmReportFilter = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFilter_MinQuotaUsage: FsrmReportFilter = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFilter_FileGroups: FsrmReportFilter = 5i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFilter_Owners: FsrmReportFilter = 6i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFilter_NamePattern: FsrmReportFilter = 7i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFilter_Property: FsrmReportFilter = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmReportFormat = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFormat_Unknown: FsrmReportFormat = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFormat_DHtml: FsrmReportFormat = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFormat_Html: FsrmReportFormat = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFormat_Txt: FsrmReportFormat = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFormat_Csv: FsrmReportFormat = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportFormat_Xml: FsrmReportFormat = 5i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmReportGenerationContext = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportGenerationContext_Undefined: FsrmReportGenerationContext = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportGenerationContext_ScheduledReport: FsrmReportGenerationContext = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportGenerationContext_InteractiveReport: FsrmReportGenerationContext = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportGenerationContext_IncidentReport: FsrmReportGenerationContext = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmReportLimit = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxFiles: FsrmReportLimit = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxFileGroups: FsrmReportLimit = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxOwners: FsrmReportLimit = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxFilesPerFileGroup: FsrmReportLimit = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxFilesPerOwner: FsrmReportLimit = 5i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxFilesPerDuplGroup: FsrmReportLimit = 6i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxDuplicateGroups: FsrmReportLimit = 7i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxQuotas: FsrmReportLimit = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxFileScreenEvents: FsrmReportLimit = 9i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxPropertyValues: FsrmReportLimit = 10i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxFilesPerPropertyValue: FsrmReportLimit = 11i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportLimit_MaxFolders: FsrmReportLimit = 12i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmReportRunningStatus = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportRunningStatus_Unknown: FsrmReportRunningStatus = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportRunningStatus_NotRunning: FsrmReportRunningStatus = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportRunningStatus_Queued: FsrmReportRunningStatus = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportRunningStatus_Running: FsrmReportRunningStatus = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmReportType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_Unknown: FsrmReportType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_LargeFiles: FsrmReportType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_FilesByType: FsrmReportType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_LeastRecentlyAccessed: FsrmReportType = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_MostRecentlyAccessed: FsrmReportType = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_QuotaUsage: FsrmReportType = 5i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_FilesByOwner: FsrmReportType = 6i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_ExportReport: FsrmReportType = 7i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_DuplicateFiles: FsrmReportType = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_FileScreenAudit: FsrmReportType = 9i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_FilesByProperty: FsrmReportType = 10i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_AutomaticClassification: FsrmReportType = 11i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_Expiration: FsrmReportType = 12i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmReportType_FoldersByProperty: FsrmReportType = 13i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmRuleFlags = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmRuleFlags_Disabled: FsrmRuleFlags = 256i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmRuleFlags_ClearAutomaticallyClassifiedProperty: FsrmRuleFlags = 1024i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmRuleFlags_ClearManuallyClassifiedProperty: FsrmRuleFlags = 2048i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmRuleFlags_Invalid: FsrmRuleFlags = 4096i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmRuleType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmRuleType_Unknown: FsrmRuleType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmRuleType_Classification: FsrmRuleType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmRuleType_Generic: FsrmRuleType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmStorageModuleCaps = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleCaps_Unknown: FsrmStorageModuleCaps = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleCaps_CanGet: FsrmStorageModuleCaps = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleCaps_CanSet: FsrmStorageModuleCaps = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleCaps_CanHandleDirectories: FsrmStorageModuleCaps = 4i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleCaps_CanHandleFiles: FsrmStorageModuleCaps = 8i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmStorageModuleType = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleType_Unknown: FsrmStorageModuleType = 0i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleType_Cache: FsrmStorageModuleType = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleType_InFile: FsrmStorageModuleType = 2i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleType_Database: FsrmStorageModuleType = 3i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmStorageModuleType_System: FsrmStorageModuleType = 100i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub type FsrmTemplateApplyOptions = i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmTemplateApplyOptions_ApplyToDerivedMatching: FsrmTemplateApplyOptions = 1i32;
#[doc = "*Required features: `\"Win32_Storage_FileServerResourceManager\"`*"]
pub const FsrmTemplateApplyOptions_ApplyToDerivedAll: FsrmTemplateApplyOptions = 2i32;
